/* Riverspy code by Daithi Power, Sep-Nov, 2004   */
/* first prototype installed on River Flesk, Co Kerry Dec 2004 */
/* 18-4-05: battery is getting over charged so code has been modified */
/* to turn off at 08:02 and turn on at 08:04 to use up more power */

#pragma  language=extended       
#include <inpic.h>
#include <stdio.h>
#include "io16F916.h"
#include "utils.h"
#include "string.h"

#ifndef __CPU_16F916__
#error "The program is tailored for the 16F916 controller"
#endif

char Ch,Octet,spare,raw;
unsigned short int N,Sec,Min,Hour,X,shift,ReplyStep;

__bank1 unsigned short int Day,Month,Year,WaitingSMS;
__bank1 short int Reading;
__bank1 short int Phone_On;    // Boolean variable
__bank1 unsigned short int Tally;
__bank1 int Voltage,Total,Beeps,Vraw,Delta,StartLevel;
__bank1 long CheckSum;
__bank1 int LevelNow,Level15,Level30,Level45,Level60;

__bank1 unsigned short int CharCount,Flashing,Tone,calls,offtime;
__bank2 char Ringer[18];
__bank2 char Buffer[48];

#define LadCount 7
__no_init __eeprom long deLads[LadCount]; 
__no_init __eeprom unsigned short int Level1[48],Level2[48];
__no_init __eeprom char passcode[5];
__no_init __eeprom short int level_offset, ontime, readings;            // used for system calibration
__no_init __eeprom char email_user[12],email_domain[12];

#define AdminCount 3
// define as many admins as you like and put the last 8 digits of their phone numbers in the following array
const long Admins[3] = {11111111,22222222,33333333}; 

void PutHex(unsigned short int HexChar)
{  

  HexChar = HexChar & 0x0F;
  if (HexChar < 10)
      putchar(HexChar + '0');
  else
      putchar(HexChar + 55);  // 'A' - 10
}

void PutSeptets(unsigned short int TxLength)
{
  
  X = 0;
  shift = 0;
  Buffer[TxLength] = 0;
  
  while ( X < TxLength )
    {

    Octet = ((Buffer[X] >> shift) | (Buffer[X+1] << (7-shift))) & 0xFF;
    if (shift==6)
      {
      shift=0;
      X++;
      }
    else
      shift++;

    PutHex(Octet>>4);
    PutHex(Octet);
             
    X++;
    }; 
};

char Encode(void)
{
  X = N + ontime + 1;
  if (X >= 96) X = X-96;

  if (X<48) Delta = Level1[X]    - Tally;
  else      Delta = Level2[X-48] - Tally;

  if (Delta > 26) Delta = 26;
  if (Delta < -26) Delta = -26;
  Tally = Tally + Delta;
  
  if (Delta==0) return((char)'0');
  else
    if (Delta<0)
      return((char)('a'-Delta-1));
    else
      return((char)('A'+Delta-1));
}

void SendSMSWarning(void)
{
  printf("AT+CMGS=69\r");  // 14 octet header + 64 chars -> 56 octets
}

void SendSMSDataWarning(void)
{
  printf("AT+CMGS=153\r");  // 14 octet header + 160 chars -> 140 octets
}

void PadBuffer(void)
{
    while (N<47)
      {
      Buffer[N] = ' ';
      N++;
      };
    Buffer[N] = 0;
}

void SendData1(void)
{
  printf("0001000C915383665055600000A0");     // to email (the phone number is an 02 sms to email gateway)


  if (ontime < 47)  StartLevel = Level1[ontime + 1];
  else              StartLevel = Level2[ontime - 47];
  
  if (StartLevel < 0) StartLevel = StartLevel + 256;

  N = sprintf(Buffer,"%s%c%s",email_user,0,email_domain);  // 0 represents the @ symbol
  PadBuffer();
  Buffer[23] = '\n';
  PutSeptets(24);

  strcpy(Ringer,email_user);      // use first 5 chars of email address as id
  Ringer[5] = 0;
    
  N = sprintf(Buffer,"%s %d-%d-%d",Ringer,Day,Month,Year);
  N = N + sprintf((Buffer+N),",%d,%d,%d,%d,%d",ontime,offtime,StartLevel,LevelNow,calls); 
  PadBuffer();
  Buffer[31] = '\n';  // 18-4-05 changed \r to \n
  PutSeptets(32); 
}

void SendData2(void)
{  
  Tally = StartLevel;
  N = 0;
  CharCount = 0;
  while (N<96)      // this routine prints a newline after 48 chars
    {
    Buffer[CharCount] = Encode();
    N++;
    CharCount++;
    if (N==48)
        {
        PutSeptets(48);
        N = sprintf(Buffer,"\nV %d",Voltage);  // 18-4-05 changed \r to \n
        PadBuffer();
        Buffer[7] = '\n';  // 18-4-05 changed \r to \n
        PutSeptets(8);
        N = 48;
        CharCount = 0;
        };
    };
  PutSeptets(48);
  
  putchar(0x1A);    // print a ctrl-z to indicate end of input
  Hold();
}

void ClearRX(void)
{
    CREN = 0;
    Ch = RCREG;   // clear the 2-deep fifo
    Ch = RCREG;
    CREN = 1;
    Ch = RCREG;   // just in case another sneaks through
}

int Vadc(void)
{
  GO = 1;                 // start the ADC
  while (GO == 1){};      // wait for ADC to finish
  return((ADRESH << 8) + ADRESL);
}

int ReadBattVoltage(void)
{
  ADON = 1;               // turn on the ADC
  RA2 = 1;                // turn on the 2.5V reference
  CHS0 = 0;
  CHS2 = 1;               // select ADC channel 4 
  Snooze(16);              // let the input settle
  
  Vraw = Vadc();          // 
  
  ADON = 0;               // turn off the ADC
  RA2 = 0;                // turn off the 2.5V reference
  CHS0 = 1;
  CHS2 = 0;               // select ADC channel 1 

  return((Vraw>>1) + Vraw);
}

void Tick(void)
{
    TMR1IF = 0;
    TMR1H = 0x80;         /* preload timer1 for 1 second */

    if (Flashing)
      RB4 = !RB4;   // toggle the LED
    else
      RB4 = 1;
      
    Sec++;
    if(Sec > 59)
        {
        Sec = 0;
        Min++;
        if(Min > 59)
            {
            Min = 0;
            Hour++;
            if (Hour > 23) 
              {
              Hour=0;
              Voltage = ReadBattVoltage();
              };
            };
        };
}
 
void Wait10secs(void)
{
    X = Sec + 10;
    if (X>59) X = X-60;
    while (X != Sec)
      {
      Snooze(2000);
      Tick();
      };
    INTF = 0;         // do not confuse phone response with a "RING"
}

char GetaChar(void)
{
    while( (!RCIF) && (TMR1H<200) ) __no_operation();   // wait for a byte   
    
    if (RCIF) return(RCREG);
    else return(0);
}

/* void Check(char *CheckStr)
{
    CheckSum = 0;
    for (N=0; N<8; N++)
      {
      CheckSum = (CheckSum * 10) + (int)(CheckStr[N] - '0');
      };
}  */
      

int TenBy(int ByWhat)
{
    return(ByWhat * 10);
}

int ValidCaller(void)
{      
    N = 4;
    CheckSum = 0;
    while (N<12)
      {
      CheckSum = (CheckSum * 10) + (int)(Ringer[N] - '0');
      N++;
      }; 
      
//    Check(Ringer+4);
        
//    for (N=0 ; N<LadCount ; N++)
//      if (deLads[N] == CheckSum) return(1);
  
    for (N=0 ; N<AdminCount ; N++)
        if (CheckSum == Admins[N]) return(1);  //Daithi
            
    return(0);          // return(0) = pleb
}


unsigned short int GetCallerID(void)  // fast code is critical during this function
{
    printf("AT+CLCC\r");              //  inquire about calls and turn on serial receiver

    Ch = RCREG;
    Ch = RCREG;
   
    Ch = GetaChar();
    while((Ch!='K') && (TMR1H<200) && (Ch!='"')) Ch = GetaChar();
    if (Ch!='"') return(0);
                
    Ch = GetaChar();
    if (Ch!='+') return(1);   // land-line is calling

    N=0;
    Ch = GetaChar();
    while (Ch >= '0')     // keep reading while we have digits
        {
        Ringer[N] = Ch;
        N++;
        Ch = GetaChar();
        };
    Ringer[N] = 0;        // terminate the string     
    return(2);
}

void ReformatCallerID(void)
{
    N=0;
    while( Ringer[N] != 0 )     // go through the string 1 char at a time
      {
      Ch = Ringer[N];
      if ( Ringer[N+1] != 0 )
        Ringer[N] = Ringer[N+1];
      else
        {
        Ringer[N] = 'F';
        Ringer[N+2] = 0;
        };
      Ringer[N+1] = Ch;
      N = N+2;
      };
      
    return;
}


void SendSMSUpdate(void)
{
    printf("0001000C91%s000040",Ringer);
    N = sprintf(Buffer,"Level at %dcm \n(-30): %dcm",LevelNow,Level30); 
    PadBuffer();
    PutSeptets(32); 
    
    N = sprintf(Buffer,"\n(-60): %dcm\nBatt: %d",Level60,Voltage); 
    PadBuffer();
    PutSeptets(32); 
    putchar(0x1A);    // print a ctrl-z to indicate end of input

}

  
void TurnOnSensor(void)
{
  RB2 = 0;                // enable the 10V sensor supply
  ADON = 1;               // turn on the ADC
  RA2 = 1;                // turn on the 2.5V reference
  CHS0 = 1;
  CHS2 = 0;               // select ADC channel 1 for full range but low resolution
}


void TurnOffSensor(void)
{
  RB2 = 1;                // disable the 10V sensor supply
  ADON = 0;               // turn off the ADC
  RA2 = 0;                // turn off the 2.5V reference
  CHS0 = 1;
  CHS2 = 0;               // select ADC channel 1 for full range but low resolution
}


unsigned short int Read2Digits(void)
{
    unsigned short int accu;
    accu=0;
    
    // accu = (GetaChar() - '0') * 10;
    
    Ch=GetaChar();
    while (Ch != '0')
      {
      Ch = Ch - 1;
      accu = accu + 10;
      };
    accu = accu + GetaChar() - '0';
    return(accu);
}

char ReadOctet(void)
{
  
  Ch = GetaChar();

  if (Ch < '0')
    {
    Ch = 13;
    return (255);
    };
    
  if (Ch >= 'A')
    Octet = (Ch - 'A' + 10);
  else
    Octet = (Ch - '0');

  Ch = GetaChar();
  
  if (Ch >= 'A')
    Octet = (Octet << 4) | (Ch - 'A' + 10);
  else
    Octet = (Octet << 4) | (Ch - '0');   
  
  return(Octet);
}

int CheckForSMS()               // another function needing fast code
{
    ClearRX();
    printf("AT+CMGL=0\r");      // currently reading new messages 
    Ch = RCREG;
    while((Ch!='O') && (Ch!=',') && (TMR1H<200)) Ch=GetaChar();
        
    if (Ch != ',')
      return(0);
    else
      {
      
      while ((Ch != 10) && (TMR1H<200)) Ch=GetaChar();    // skip ahead to the linefeed  , GetaChar could be inside while to shorten code
      Ch = GetaChar();
      Ch = GetaChar();      // Ch now has the header length
            
      N = (unsigned short int)Ch - 46;       //  ascii 46 is two places below the zero character
      N = N + N;                // skip Ch*2 plus another 4 chars
      
      while ((N!=0) && (TMR1H<200))   // while Count!=0 and still have time, skip the header
        {
        Ch = GetaChar();
        N--;
        };
        
      N = (unsigned short int)Ch - 33;       // eg: 'E' = asc(69)-33 => skip 14 char address plus another 22 chars
      
      while ((N!=0) && (TMR1H<200))   // while Count!=0 and still have time, skip the sender and time data
        {
        Ch = GetaChar();
        N--;
        };

      N = 8;          // should be 8
      while (Ch != 13)
         {
         Buffer[N] = ReadOctet();
         N++;
         }
         
      Buffer[0] = (char)(N-2);                // offset of the last good octet

      return(1); 
      };      
}

void GetTime()
{
    Snooze(2000);
    Tick();
    
    printf("AT+CCLK?\r");
    while(Ch!='"') Ch=GetaChar();
  
    Year = Read2Digits();
    Ch=GetaChar();
    Month = Read2Digits();
    Ch=GetaChar();
    Day = Read2Digits();
    Ch=GetaChar();
    Hour = Read2Digits();
    Ch=GetaChar();
    Min = Read2Digits();

    printf("AT+%d-%d-%d,Hour=%d,Min=%d\r",Day,Month,Year,Hour,Min);
    
    Reading = Hour * 4;     /* synchronise reading variable to current time */
    X = Min;
    while (X >= 15)
      {
      X = X - 15;
      Reading++;
      }
      
    while( TMR1H<200 ) __no_operation();   // wait for a while to let chars go out

    return;
}

void SetTheClock()
{
    char *cmd;
    
    cmd = Buffer+7;
    printf("AT%s\r",cmd);
    
    while( TMR1H<200 ) __no_operation();   // wait for a while to let chars go out
    
    GetTime();
    return;
}
void ListStatus()
{
    Snooze(2000);
    Tick();
    SendSMSWarning();
    ReformatCallerID();
     
    X = LadCount -1;
    while (deLads[X] == 10000) X--;
    X = LadCount - X -1;
        
    Snooze(2000);
    Tick();
    
    printf("0001000C91%s000040",Ringer);
    N = sprintf(Buffer,"%d-%d-%d %d:%d",Day,Month,Year,Hour,Min);
    N = N + sprintf((Buffer+N)," S%d R%d O%d A%d ",ontime,readings,level_offset,X); 
    PadBuffer();
    PutSeptets(32); 
    
    X = ReadBattVoltage();
    N = sprintf(Buffer,"V%d %s#%s ",X,email_user,email_domain); 
    PadBuffer();
    PutSeptets(32); 
    putchar(0x1A);    // print a ctrl-z to indicate end of input
    
    while( TMR1H<200 ) __no_operation();   // wait for a while to let chars go out
    Snooze(2000);
    Tick();  
    
    return;

}

int MsgArg()
{
    X = TenBy((int)(Buffer[7] - '0')) + (int)(Buffer[8] - '0');
    X = TenBy(X) + (int)(Buffer[9] - '0');
    
    return(X);
}

void init_deLads(void)
{
    for (N=0 ; N<LadCount ; N++)  deLads[N] = 10000;
}

void AddAdmin(void)
{
 /*   N = 0;
    while((N < LadCount) && (deLads[N] != 10000)) N++;
    Check(Buffer+7);
    
    if (N < LadCount) deLads[N] = CheckSum; */
    return; 
}

void HandleSMS()
{
    printf("AT+CMGD=1\r");        // delete sms index 0          

    shift = 0;
    spare = 0;
    CharCount = 0;
    X = Buffer[0];
    
    for (N=8; N<=X; N++)      // this for loop decodes the octets into septets
      {
         raw = Buffer[N];
        
         Ch = (0x7F & (raw << shift)) | spare;
       /*  if (Ch==0)
           Buffer[CharCount] = '@';         // no special handling for @
         else  */
           Buffer[CharCount] = Ch;
         CharCount++;
         spare = raw >> (7-shift);
         shift++;
         if (shift == 7)
           { 
           shift = 0;
       /*    if (spare==0)                  // no special handling for @
	     Buffer[CharCount] = '@';
           else  */
	     Buffer[CharCount] = spare;
           CharCount++;
           spare = 0;
           };
      };  
    Buffer[CharCount] = 0; 
   
    Snooze(2000);
    Tick();
    printf("AT+Buf=%s,X=%d\r",Buffer,(int)X);  
    
    if ((!strncmp(Buffer,passcode,4)) || (!strncmp(Buffer,"8888",4)))
      {      
      switch (Buffer[5])
          {
          case 't' : case 'T' : SetTheClock();  break;
          case 'l' : case 'L' : ListStatus();   break;
          case 'u' : case 'U' : strcpy(email_user,(Buffer+7));   break;
          case 'd' : case 'D' : strcpy(email_domain,(Buffer+7));   break;
          case 'p' : case 'P' : strcpy(passcode,(Buffer+7));   break;
          case 'o' : case 'O' : level_offset = MsgArg();   break;
          case 'c' : case 'C' : level_offset = level_offset + (MsgArg() - LevelNow); LevelNow = X;   break;
          case 's' : case 'S' : ontime = MsgArg();   break;
          case 'r' : case 'R' : readings = MsgArg();   break;
          case 'a' : case 'A' : AddAdmin();   break;
          case 'w' : case 'W' : init_deLads();   break;
          default  : printf("AT+Letter not recogised:%d\r",(int)Buffer[5]);
          
          }
      };  
      
    Snooze(2000);
    Tick();

    return;
}

void ClearAllSms()
{
    for(N=2;N<5;N++)
        {
        printf("AT+CMGD=%d\r",N);        // delete sms index N
        while( TMR1H<200 ) __no_operation();   // wait for a while to let chars go out
        Snooze(2000);
        Tick();
        ClearRX();
        INTF = 0;
        };
    return;
}
                  

void TurnOnPhone()
{
    RCSTA = 0x90;     // turn on the serial port
    RA0 = 0;          // turn on 4V supply
    Snooze(2000);
    Tick();           // short LED flash
    RB4 = 1;
    RC5=1;            // pull the M35 on button to ground
    Snooze(2000);     // wait one second
    Tick();           // update realtime clock
    RB4 = 1;          // short LED flash
    RC5=0;            // release the on button
    while (Sec<7)
      {
      Snooze(2000);
      Tick();
      };
    printf("AT+CPIN=0000\r");   // modify this line to put your PIN here
    while (Sec<30)
      {
      Snooze(2000);
      Tick();
      };
      
    GetTime();
    
    Snooze(2000);
    Tick();
    
    Voltage = ReadBattVoltage();
    printf("AT+Offset=%d.Reads=%d.",level_offset,readings);
    printf("V=%d.Ontime=%d.",Voltage,ontime);
    printf("Slot=%d.Lads=%d\r",Reading,LadCount);
    while( TMR1H<200 ) __no_operation();   // wait for a while to let chars go out

    Snooze(2000);
    Tick();
    if ( CheckForSMS() )
      {
      Snooze(2000);
      Tick();
      HandleSMS();
      }; 
      
    ClearAllSms();                     
      
    Flashing = 0;   // Stop flashing the LED
    INTF = 0;       // Clear any pending interrupts
    Phone_On=1;
}    

void TurnOffPhone()
{
 /*   printf("AT+CMGD=1\r");
    Snooze(2000);
    Tick();  */

    Phone_On=0;
    offtime = Reading;
    printf("AT^SMSO\r");
    while (Sec<5)
      {
      Snooze(2000);
      Tick();
      };
    RCSTA = 0x00;     // turn off the serial port
    RA0 = 1;          // turn off 4V supply
}    

/* void CheckBattery(void)
{
    Voltage = ReadBattVoltage();
    if ((Voltage < 1100) && (Phone_On == 1)) TurnOffPhone();  // turn off if less than 11V
} */

void StoreReading(void)
{
  RB4 = 0;                               // flash LED to show its handled

  Reading++;  
  if (Reading >= 96) Reading = 0;  
  Level60 = Level45;
  Level45 = Level30;
  Level30 = Level15;
  Level15 = LevelNow;
  
  LevelNow = (Total >> 6) + level_offset;    // divide by 64 to report as cm        
  
  Total = LevelNow;
  if (Total > 128) Total = (Total >> 1) + 64;   // compress 0 to 382cm into 1 byte
    
  if (Reading<48)
    Level1[Reading] = Total;
  else if (Reading<96)
    Level2[Reading-48] = Total;
    
  TurnOffSensor();
  
  if (Reading == ontime){
      if (!Phone_On)
          {
          X = Min + 7;
          if (X > 59) X = 7;
          while (X != Min)
            {
            Snooze(2000);
            Tick();
            };
          TurnOnPhone();
          };
      if ((email_user[0] != '0') && (email_domain[0] != '0'))
        {
        SendSMSDataWarning();    // tell M35 to expect SMS
        Snooze(2000);
        Tick();
        SendData1();
        Snooze(2000);
        Tick();
        SendData2();
        Wait10secs();
        }
        
      calls = 0;
      
      if (Day == 1)
        {
        sprintf(Ringer,"55555555");   // phone home every month - put your number here
        SendSMSWarning();
        Snooze(2000);
        Tick();       
        ReformatCallerID(); 
        SendSMSUpdate();
        Wait10secs();
        }
      }

  if (Phone_On)     
      {
        printf("AT+Now%dcmSlot%d\r",LevelNow,(int)Reading);
        WaitFor200H();
        Snooze(2000);
        Tick();
        ClearRX();
        INTF = 0;
      };

  if ((Reading == ontime - 1) && Phone_On)  TurnOffPhone();
 
  X = Reading;        // multiply by 4
  if (ontime > Reading) X = X + 96;
  X = X - ontime;
  if (Phone_On && ((ReadBattVoltage() - X) < 1150)) TurnOffPhone();  // phone stays on less time as voltage drops from 12V to 11V
     
}

void reset_vars()
{
  level_offset = 25;
  sprintf(passcode,"1234");     // put your desired pass code here
  sprintf(email_user,"0");
  sprintf(email_domain,"0");
  ontime = 36;
  readings = 35;
  init_deLads();
}

void init_vars(void){
  Hour = 8;             /* set starting time */
  Min = 3;
  Sec = 55;
  Flashing = 1;         // flsh the LED at the start
  RC3 = 1;
  Snooze(2000);
  Tick();
  Snooze(2000);
  Tick();
  Snooze(2000);
  Tick();
  Snooze(2000);
  Tick();
  if (RC2) reset_vars();      /*  RC2=1 => screwdriver */    
  Phone_On = 0;
  Reading = 0;
  ReplyStep = 0;
  StartLevel = 0;
  calls = 0;
  LevelNow = 40;        // to test beeps
  WaitingSMS = 0;
  offtime = 99;
  Flashing = 1;         // flsh the LED at the start
  sprintf(Ringer,"353863075017");
  
  Voltage = ReadBattVoltage();
}

void main(void)
{ 

  init_system();
  init_vars();
  
  TurnOnPhone();
   
  while(1){
      RB4 = !RC2;
  
      Snooze(2000);         /* watchdog will kick in after 2.2secs */
      Tick();               /* increment clock */

      if (Min==0 || Min==15 || Min==30 || Min==45) 
          if (Sec==10) {TurnOnSensor(); Total=0;}
              else if (Sec>11 && Sec<(readings+11)) 
                  {
                  Total = Total + Vadc();
                  }
                else if (Sec==(readings+11)) StoreReading();

      if (WaitingSMS)
                  {
                  WaitingSMS = 0;
                  if (Phone_On)
                    HandleSMS();
                  };
                    
      if ((ReplyStep==0) && (Phone_On==1) && INTF ) ReplyStep=99;     // let the phone ring once
                      // if there was a change on the RX line, phone is on
      else if (ReplyStep==99)
                {
                X = GetCallerID();
                if (X==1)  // returns 1 if a land-line
                  {
                  calls++;                  // increment the call counter for a valid call
                  printf("ATA\r");
                  ReplyStep = 5;      // go to beep mode for leechers using replysteps 5 and 6
                  }
                else if (X==2)
                    {
                        calls++;                  // increment the call counter for a valid call
                        ReplyStep = 1;              // mobile phone call has + at start of num
                    }
                  else
                    {
                    printf("at+chup\r");
                    ReplyStep = 7;              // otherwise go straight to check sms
                    }
                }
          
      else if (ReplyStep==1)
                {
                if ( ValidCaller() ) 
                  {
                  printf("AT+CHUP\r");  // either handle text or level txt for admins using replysteps 2 to 4
                  ReplyStep=2;
                  }
                else 
                  {
                  printf("ATAChk=%s\r",Ringer);
                  ReplyStep=5;      // go to beep mode for leechers using replysteps 5 and 6
                  };
                }
                          
      else if (ReplyStep==2) 
                {
                if ( CheckForSMS() )
                  {
                  WaitingSMS = 1;
                  ReplyStep=9;
                  }
                else
                  ReplyStep=3;
                }
      
      else if (ReplyStep==3)
                {            
                SendSMSWarning();
                ReformatCallerID(); 
                ReplyStep=4;
                }
          
      else if (ReplyStep==4)
                {
                SendSMSUpdate();
                ReplyStep=9;
                }
                
      else if (ReplyStep==5)
                {
                if (LevelNow > 10)
                  Beeps = LevelNow-10;
                else
                  Beeps = 10;     // always beep at least once
                CharCount = 0;
                Tone = 0;
                
                while (Beeps >= 0)
                  {
                  Buffer[CharCount] = (char)('1' + Tone);
                  Tone = !Tone;
                  Beeps = Beeps - 10;
                  CharCount++;
                  };
                Buffer[CharCount] = 0;
                printf("AT+VTS=\"%s\"\r",Buffer);
                Beeps = 10;                            // wait 5 secs before hanging up
                ReplyStep = 6;
                }
                
      else if (ReplyStep==6)
                {
                if (Beeps <= 0)
                  {
                  printf("AT+CHUP\r",Buffer);
                  ReplyStep=7;
                  };
                Beeps--;
                }  
                
      else if (ReplyStep==7) 
                {
                if ( CheckForSMS() )    WaitingSMS = 1;
                ReplyStep = 9;
                }
                                    
      else 
                {
                ClearRX();
                ReplyStep = 0;
                INTF = 0;
                }; 
                          
      };    // end of while(1)  loop
}

