#pragma language=extended       
#include <inpic.h>
#include "io16F916.h"

void init_system(void)
{
  __set_configuration_word(INT_OSC_NOCLKOUT & BOR_SLEEP & CP_OFF & CPD_OFF & WDT_ON & PWRTE_ON & DEBUG_OFF);
  OSCCON = 0x40;        /* set clock frequency to 1MHz */
  OSCTUNE = 0xF6;       /* adjust to 922kHz   0xF5 gave 19000 baud 0x00 gave 20800*/
  CMCON0 = 0x07;        /* disable the comparator module */
  VLCDEN = 0;           /* disable lcd voltage */
  PORTA = 0x00;
  PORTB = 0x0A;
  TRISA = 0xBA;         /* set all except RA0, RA2 ,RA6 as inputs */
  TRISB = 0xEB;         /* set all except RB2, RB4 as inputs*/
  TRISC = 0x97;         /* set all except RC5,RC6 as inputs*/
  OPTION = 0xB7;        /* set PSA=0 to use watchdog prescaler , RB0 falling edge */
  WDTCON = 0x11;        /* turn on watchdog, set timer to 250ms */
  T1CON = 0x0F;         /* set timer1 for 32kHz real time clock */
  
  INTCON = 0xC0;        /* enable global interrupts and PEIE */
  PIE1 = 0x01;          /* enable timer 1 interupt */
  PIE2 = 0x00;             /* disable other peripheral interrupts */  
  
  SPBRG = 2;               // set 19200 baud at 922kHz clock
  TXSTA = 0x24;             // 8-bit transmission, async mode, high baud
  RCSTA = 0x90;             // 8-bit reception, disable serial port initially

  ANSEL = 0x1A;             // Use channels 1,4 and 3 for ref
  ADCON0 = 0xA4;            // Use Vref+,Vss - Right justify result, CH1, off for now
  ADCON1 = 0x50;             // use Fosc/16 
  
}


void Hold(void)
{
    int x;
    x=0;
    while (x<200) { __no_operation(); x++; };
}

void WaitFor200H(void)
{
    while( TMR1H<200 ) __no_operation();   // wait for a while to let chars go out
}

void Snooze(const int Time_ms)
{
    Hold();
    
    if (Time_ms==16)
      WDTCON = 0x09; 
    else
      WDTCON = 0x17;   /* turn on watchdog, set timer to 2s */   
    __clear_watchdog_timer();
    
    __sleep();
    WDTCON = 0x17;        /* set default time to 2s */
}

